/* jmixer.cpp - part of MUSE
 * (c)2000 Denis Roio aka jaromil
 * c++ engine for multiple mp3 realtime stream mixing
 */

#include <iostream.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <termios.h>
#include <sys/time.h>
#include <config.h>

#include <jutils.h>
#include "jmixer.h"
#include "audioproc.h"

/* process_buffer BUF_SIZE is:
   BUF_SIZE of 32bit ints *2channels *8resampling_space

   audio_buffer BUF_SIZE is:
   BUF_SIZE of 16bit short *2channels *8resampling_space
*/

Stream_mixer::Stream_mixer() {

  /* well... let's initialize channels with mp3 decoders */
  int i;
  for(i=0;i<MAX_CHANNELS;i++)
    xach[i] = NULL;

  dsp = 0;
  have_gui = false;

  dspout = false;
  icecastout = false;
  fileout = false;
  quit = false;

  if(pthread_mutex_init (&_mutex,NULL) == -1)
    error("error initializing POSIX thread mutex");
  if(pthread_cond_init (&_cond, NULL) == -1)
    error("error initializing POSIX thread condtition"); 
}

Stream_mixer::~Stream_mixer() {
  int i;

  if(dsp!=0)
    close_soundcard();
  
  if(icecastout)
    stop_icecast();

  for(i=0;i<MAX_CHANNELS;i++)
    delete xach[i];
  
  if(pthread_mutex_destroy(&_mutex) == -1)
    error("error destroying POSIX thread mutex");
  if(pthread_cond_destroy(&_cond) == -1)
    error("error destroying POSIX thread condition");
}

void Stream_mixer::register_gui(Amuse *reg_gui) { 
  gui = reg_gui; 
  have_gui = true;
  if(HAVE_LAME) {
    char temp[256];
    lame.version();
    sprintf(temp,"Encoding with Lame %s",lame.ver);
    gui->status_lame->value(temp);
  }
}

bool Stream_mixer::open_soundcard()
{
    int format,tstereo,speed,caps,val;
    
    if((dsp=open("/dev/dsp",O_RDWR))==-1) {
        perror("Error opening /dev/dsp: ");
	return(false);
    } else
      notice("Found soundcard on /dev/dsp");

    /* BUFFER FRAGMENTATION */
    val = (FRAGCOUNT << 16) | FRAGSIZE;

    if(ioctl(dsp,SNDCTL_DSP_SETFRAGMENT,&val)==-1)
      error("failed to set dsp buffers:");

    /* or LE */
    format = AFMT_S16_LE;

    tstereo = 1; /* only stereo _DSP_ in/out */
    speed = SAMPLE_RATE; /* 44100hz mixing */

    ioctl(dsp, SNDCTL_DSP_GETCAPS, &caps);
    if(caps & DSP_CAP_DUPLEX) {
      fullduplex = true;
      act("full duplex supported. good");
      ioctl(dsp, SNDCTL_DSP_SETDUPLEX, 0);
      ioctl(dsp, DSP_CAP_DUPLEX, 0);
    } else {
      act("full duplex is not supported");
      fullduplex = false;
    }

    /* FORMAT */
    if(ioctl(dsp,SNDCTL_DSP_SETFMT,&format)==-1)
      error("failed to set data format");

    /* CHANNELS */
    if(ioctl(dsp,SNDCTL_DSP_STEREO,&tstereo)==-1)
      error("something went wrong with the stereo setting");

    /* SAMPLERATE */
    if(ioctl(dsp,SNDCTL_DSP_SPEED,&speed)==-1)
      error("speed setting failed");

    /* GET FRAG SIZE BACK */
    if(ioctl(dsp,SNDCTL_DSP_GETBLKSIZE,&val)==-1)
      error("get block size failed to return");

    act("mixing 16bit %dHz stereo",speed);

    dspout = true;

    livein = new LiveIn(speed, tstereo+1, &dsp);

    return(true);
} /* open_soundcard */

bool Stream_mixer::set_icecast(char *ip, int port, char *pass, char *mountpoint, int bps, char *name, char *url, char *desc, int login) {
  lock();

  if(!ice.set_server(ip,port,pass,mountpoint,bps,login)) {
    if(have_gui) gui->status_main->value("doom damnit! i can't resolve icecast hostname");

    error("Can't resolve %s",ip);

    unlock();
    return(false);
  }
  ice.set_meta(name, url, desc);
      
  if((!lame.initialized)||(lame.bps!=bps)) {
    gui->status_main->value("ouch! lame is not initialized!");

    unlock();
    return(false);
  }

  /* ::::: we are connected */
  if(ice.start()) {
    /* notice the user */
    if(have_gui) {
      char temp[256];
      gui->status_main->value("WE GOT MILK !!");
      sprintf(temp,"STREAMING ON [ http://%s:%u/%s ]",ip,port,mountpoint);
      gui->status_ice->value(temp);
    } else
      fprintf(stderr,"Estabilished %ubps stream on %s:%u/%s\n",bps,ip,port,mountpoint);

    icecastout = true;

  }
  /* ::::: connection failed */
  else {
    
    icecastout = false;
    if(have_gui) {
      char temp[256];
      sprintf(temp,"Got problems streaming on server %s. No milk??",ip);
      gui->status_main->value(temp);
    }
  }
  
  unlock();
  return(icecastout);
}

void Stream_mixer::set_lame(int bps, int freq, int mode, int quality, int lowpass, int highpass) {
  lock();
  lame.init(bps, freq, mode, quality, lowpass, highpass);
  unlock();
}

void Stream_mixer::stop_icecast() {
  lock();
  ice.stop();
  icecastout = false;
  unlock();

  if (have_gui) {
    gui->status_main->value("Stopped streaming to icecast server");
    gui->status_ice->value("");
  } else
    fprintf(stderr,"Stopped streaming to icecast server\n");
}

int Stream_mixer::set_mp3file(char *file, int bps) {
  int res;
  
  lock();
  res = mp3file.openmp3(file);
  if(res>0) fileout = true;
  unlock();
  /*  
      if((!lame.initialized)||(lame.bps!=bps))
      lame.init(bps);
  */
  return(res);
}

void Stream_mixer::stop_mp3file() {
  lock();
  mp3file.closemp3();
  fileout = false;
  unlock();
}

void Stream_mixer::cafudda()
{
  int i;
  float res;
  int max = 0;

  if(quit) return;

  /* here memset takes byte num */
  memset(process_buffer,0,PROCBUF_SIZE<<(SIZEOF_INT>>1));

  lock();

  for(i=0;i<MAX_CHANNELS;i++) {
    if(xach[i] != NULL) {
      if(xach[i]->on) {

	res = xach[i]->mix(process_buffer);
	max = (max<xach[i]->in_smp) ? xach[i]->in_smp : max;
	updchan(i,res);
	xach[i]->signal();
      } /* if(xach[i].on) */
    } /* if(xach[i] != NULL) */
  } /* for(i=0;i<MAX_CHANNELS;i++) */
  
  if(livein->on) {
    int ires;
    ires = livein->mix(process_buffer);
    max = (max<ires) ? ires : max;
  }
  
  /* here: max = number of 32bit samples in process_buffer
     number of single 16bit stereo samples (max>>1)
  */

  if(max>0) {
    /* we got some sound to process */

    /* CLIPPING
       this brings it back to a 16bit resolution
       i'm using a simple as effective alghoritm (audioproc.cpp) */
    clip_audio(&process_buffer[0],&audio_buffer[0],max);

    /* ENCODE AUDIO
       here lames need the number of BYTES
       so max*channels>>1 (16bit)
       (we mix in stereo domain so channels is always 2) */
    if(fileout||icecastout) {
      lame.encode(&audio_buffer[0],max>>2);
    }
    
    /* WRITE 2 MP3 FILE */
    if(fileout) {
      mp3file.writemp3();
    }

    /* SEND 2 ICECAST */
    if(icecastout)
      if(ice.send()<0)
	if(have_gui)
	  gui->status_main->value("Send error: cannot send stream to icecast server");

    unlock();

    /* WRITE 2 DSP */
    if(dspout) {
      /* write out interleaved stereo 16bit pcm 
	 dsp takes number of samples, the format
	 is being setted with ioctls in initialization
      */
      write(dsp,audio_buffer,max);
    }
  } else {
    /* there was no sound coming in from any channel */
    wait();
    unlock();
  }
  
  /* compute and draw levels
     notice the gui to refresh */
  if(have_gui)
    if(gui->mix_monitor->showit)
      gui->mix_monitor->draw_levels( get_level(audio_buffer,max) );

}

bool Stream_mixer::create_channel(int ch) {
  lock();

  /* paranoia */
  if(xach[ch]!=NULL)
    delete_channel(ch);

  /* by default we create a mpeg channel */
  xach[ch] = new MpegChannel();

  unlock();
  
  if(xach[ch]==NULL)
    return(false);

  return(true);
}

void Stream_mixer::delete_channel(int ch) { 
  lock();
  if(xach[ch]->on) xach[ch]->stop();
  if(xach[ch]->opened) xach[ch]->clean();
  if(xach[ch]->running) {
    /* be sure it quitted */
    xach[ch]->signal();
    xach[ch]->lock(); xach[ch]->unlock();
  }

  delete xach[ch];
  xach[ch] = NULL;

  unlock();
}

void Stream_mixer::pause_channel(int ch) {
  /* here i don't lock - c'mon, boolean _is_ atomical */
  if(xach[ch]->opened) {
    xach[ch]->on = !xach[ch]->on;
  }
}

/* set_channel returns:
   0 - error
   1 - bitstream opened (seekable)
   2 - bitstream opened (non seekable)
*/
int Stream_mixer::set_channel(int ch, int playlist_pos) {
  int res;
  char *sel = playlist[ch].song(playlist_pos);
  if(sel==NULL) return 0;
  char temp[128];

  if(sel==NULL)
    return(false);

  lock();

  if(strstr(sel,".ogg")!=NULL) {
#ifdef HAVE_VORBIS
    /* take care to delete the old channel if present*/
    if(xach[ch]!=NULL) {
      if(xach[ch]->type != OGGCHAN) {
	delete xach[ch];
	/* better to wait a bit after deleting */
	jsleep(0,100);
	xach[ch] = new OggChannel();
      }
    }
#else
    warning("can't open file: OggVorbis support not compiled.");
    return(false);
#endif
  } else if(strstr(sel,".mp3")!=NULL) {
    /* take care to delete the old channel if present*/
    if(xach[ch]!=NULL) {
      if(xach[ch]->type != MP3CHAN) {
	delete xach[ch];
	jsleep(0,100);
	xach[ch] = new MpegChannel();
      }
    }
  }

  unlock();

  /* ok, we have the channel here, let's load the bitstream */
  res = xach[ch]->set(playlist[ch].song(playlist_pos));
  switch(xach[ch]->playmode) {
  case 0: /* LOOP :::::::::::::::::::::::: */
  case 1: /* CONT :::::::::::::::::::::::: */
    play_channel(ch);
    break;
  default: /* PLAY :::::::::::::::::::::::: */
    break;
  }    

  if(res==0) {
    if(have_gui) {
      gui->status_main->value("Doom damnit! can't open the file selected");
    } else {
      error("error opening file %s\n", playlist[ch].song(playlist_pos));
    }
  } else {
    playlist[ch].select(playlist_pos);
    if(have_gui) {
      switch(xach[ch]->type) {
      case MP3CHAN:
	sprintf(temp,"Mpeg on chan[%u] %ukbit %ukhz %s"
		, ch, xach[ch]->bitrate, xach[ch]->samplerate,
		(xach[ch]->channels==1) ? "mono" : "stereo" );
	break;
      case OGGCHAN:
	sprintf(temp,"OggVorbis on chan[%u] %ukhz %s"
		, ch, xach[ch]->samplerate,
		(xach[ch]->channels==1) ? "mono" : "stereo" );
	break;
      }
      gui->status_main->value(temp);
    }
  }
  return(res);
}

void Stream_mixer::set_all_volumes(float *vol) {
  int ch;
  lock();
  for(ch=0;ch<MAX_CHANNELS;ch++) {
    if(xach[ch]!=NULL)
      xach[ch]->volume = vol[ch];
  }
  unlock();
}

void Stream_mixer::set_volume(int ch, float vol) {
  lock();
  xach[ch]->volume = vol;
  unlock();
}

void Stream_mixer::set_speed(int ch, int speed) {
  lock();
  xach[ch]->speed = speed;
  unlock();
}

void Stream_mixer::play_channel(int ch) {
  if(xach[ch]->running && !xach[ch]->on) {
    lock();
    if(!xach[ch]->play())
      error("can't play channel %u",ch);
    unlock();
  }
}
  
bool Stream_mixer::stop_channel(int ch) {
  bool res = false;
  if(xach[ch]->running) {
    lock();
    res = xach[ch]->stop();
    unlock();
  }
  return(res);
}

bool Stream_mixer::set_position(float pos, int ch) {
  bool res = false;
  if(xach[ch]->seekable && xach[ch]->running) {
    lock();
    res = xach[ch]->pos(pos);
    unlock();
    if(!res) error("can't seek bitstream on channel [%u]",ch);
  }
  return(res);
}

bool Stream_mixer::set_live(bool stat) {
  if(!((dspout)&&(!fullduplex)&&(stat))) {
    lock();
    livein->on = stat;
    unlock();
  }
  return(livein->on&stat);
}

bool Stream_mixer::set_lineout(bool stat) {
  if(!((livein->on)&&(!fullduplex)&&(stat))) {
    lock();
    dspout = stat;
    unlock();
  }
  return(dspout&stat);
}

void Stream_mixer::close_soundcard() {
  ioctl(dsp, SNDCTL_DSP_RESET, 0);
  close(dsp);
}

bool Stream_mixer::set_playlist(char *file, int ch) {
  bool res = false;
  lock();
  res = playlist[ch].load(file);
  unlock();
  if(res) {
    if(have_gui) {
      gui->chan[ch]->playlist_browser->clear();
      int c;
      char *p;
      for(c=1;c<=playlist[ch].length;c++) {
	p = playlist[ch].song(c);
	if(p==NULL) return false;
	/* takes out the directory path if it's a file and
	   not a stream */
	if(strncmp(p,"http://",7)!=0) {
	  while(*p!='\0') p++;
	  while(*p!='/') p--;
	}
	gui->chan[ch]->playlist_browser->add(p+1,NULL);
      }
    }
  }
  return(res);
}

bool Stream_mixer::add_to_playlist(char *file, int ch) {
  lock();
  playlist[ch].add(file);
  unlock();
  
  if(have_gui) {
    char *p = file;
    /* takes out the directory path if it's a file and
       not a stream */
    if(strncmp(p,"http://",7)!=0) {
      while(*p!='\0') p++;
      while(*p!='/') p--;
      p++;
    }
    gui->chan[ch]->playlist_browser->add(p,NULL);
  }
  return(true);
}

void Stream_mixer::upd_timeline(int ch, float res) {
  char temp[24];
  
  if(xach[ch]->update) {
    gui->chan[ch]->timeline_slider->value(xach[ch]->time.f);
    
    sprintf(temp,"%02u:%02u:%02u",xach[ch]->time.h,xach[ch]->time.m,xach[ch]->time.s);
    gui->chan[ch]->time->value(temp);
  }

}

/* updchan takes care that any action is taken (channel ends etc.)
   and updates the gui if registered.
   this is called only by cafudda while the mixer is locked, so no need to lock
*/
void Stream_mixer::updchan(int ch, float res) {

  if(res==3.0) {
    char temp[64];
    warning("uops! channel [%u] out of sync",ch);
  } else if(res==2.0) {

    unlock();

    switch(xach[ch]->playmode) {
    case 0: /* LOOP :::::::::::::::::::::::: */     
      xach[ch]->set(playlist[ch].selected->name);
      if(have_gui) {
	gui->chan[ch]->timeline_slider->value(0.0);
	gui->chan[ch]->time->value("00:00:00");
      }
      xach[ch]->play();
      break;
    case 1: /* CONT :::::::::::::::::::::::: */
      bool lastsong;
      int newsong;
      newsong = gui->chan[ch]->playlist_browser->value()+1;
      if(newsong>playlist[ch].length) lastsong = true;
      else lastsong = false;
      
      if(lastsong) {
	playlist[ch].select(1);
	gui->chan[ch]->playlist_browser->value(1);
	gui->chan[ch]->name->value(gui->chan[ch]->playlist_browser->text(1));
      } else {
	playlist[ch].select(newsong);
	gui->chan[ch]->playlist_browser->value(newsong);
	gui->chan[ch]->name->value(gui->chan[ch]->playlist_browser->text(newsong));
      }
      xach[ch]->set(playlist[ch].selected->name);
      gui->chan[ch]->timeline_slider->value(0.0);
      gui->chan[ch]->time->value("00:00:00");
      xach[ch]->play();
      break;
    case 2: /* PLAY :::::::::::::::::::::::: */
      xach[ch]->set(playlist[ch].selected->name);
      if(have_gui) {
	gui->chan[ch]->timeline_slider->value(0.0);
	gui->chan[ch]->time->value("00:00:00");
      }
      break;
    }

    lock();

  } else {
    if(have_gui)
      upd_timeline(ch, res);
  }
}
