/* muse.cpp - MuSE
   (c)2000 Denis Roio aka jaromil
   see COPYING for licensing stuff

   here command line is parsed
   synchronous threads are launched depending on mode selection
*/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <jutils.h>
#include "config.h"
#include "generic.h"
#include "jmixer.h"
#include "gui_muse.h"
#include "gui_callbacks.h"

/* command line stuff */

char *version =
"%s version %s [ http://muse.dyne.org ]";

char *authors =
"(c)2000-2001 jaromil & august black";

char *help =
"Usage: muse [options]\n\n"
"options:\n"
"-h --help\tthis help\n"
"-v --version\tversion information\n"
"-d --debug [1-3]\tdebug verbosity level  - default 3\n"
/* soon to come
"-b --bitrate\tencoding bitrate in Kbit/s - default 32\n"
"-q --quality\tencoding quality (1-9)     - default 4\n"
"-f --filedump\tdump stream to file\n"
"-g --nogui\trun without graphic interface\n"
"-i --live\trun listening on live input\n"
"-l --playlist\trun playling a playlist\n"
"-s --server\tstream to server\n"
"-p --port\tserver port\n"
"-m --mountpoint\tmounpoint on server\n"
"-x --pass\tencoder password on server\n"
"-n --name\tname of the stream\n"
*/
"\n";

static const struct option long_options[] = {
  { "help", no_argument, NULL, 'h' },
  { "version", no_argument, NULL, 'v' },
  { "debug", required_argument, NULL, 'd' },
  /* soon to come
  { "bitrate", required_argument, NULL, 'b' },
  { "quality", required_argument, NULL, 'q' },
  { "filedump", required_argument, NULL, 'f' },
  { "nogui", no_argument, NULL, 'g' },
  { "live", no_argument, NULL, 'i' },
  { "playlist", required_argument, NULL, 'l' },
  { "server", required_argument, NULL, 's' },
  { "port", required_argument, NULL, 'p' },
  { "mountpoint", required_argument, NULL, 'm' },
  { "pass",required_argument, NULL, 'x' },
  { "name",required_argument, NULL, 'n' },
  */
  { 0, 0, 0, 0 }
};

/* char *short_options = "hvd:b:q:f:gil:s:p:m:x:n:"; */
char *short_options = "hvd:";

/* misc settings */
int debug = 1;
bool daemon_mode = false;
char *queue_file = NULL;
/*
int brate = 0;
int quality = 4;
char *filedump = NULL;
bool nogui = false;
bool live = false;
char *playlist = NULL;
char *server = NULL;
int port = 8000;
char *mountpoint = NULL;
char *pass = NULL;
char *name = NULL;
*/
void take_args(int argc, char **argv) {
  int res;

  notice(version,PACKAGE,VERSION,PACKAGE);
  act(authors);
  set_debug(3);
  
  do {
    res = getopt_long(argc, argv, short_options, long_options, NULL);

    switch(res) {
    case 'h':
      fprintf(stderr,"%s",help);
      exit(0);
      break;
    case 'v':
      exit(0);
      break;
    case 'd':
      debug = atoi(optarg);
      break;
      /*
    case 'b':
      brate = atoi(optarg);
      break;
    case 'q':
      quality = atoi(optarg);
      break;
    case 'f':
      filedump = strdup(optarg);
      break;
    case 'g':
      nogui = true;
      break;
    case 'i':
      live = true;
      break;
    case 'l':
      playlist = strdup(optarg);
      break;
    case 's':
      server = strdup(optarg);
      break;
    case 'p':
      port = atoi(optarg);
      break;
    case 'm':
      mountpoint = strdup(optarg);
      break;
    case 'x':
      pass = strdup(optarg);
      break;
    case 'n':
      name = strdup(optarg);
      break;
      */
    default:
      break;
    }
  } while(res > 0);

}

int main(int argc, char **argv) {

  take_args(argc, argv);
  set_debug(debug);
  act("starting engine...");
  
  Stream_mixer *mix = new Stream_mixer();
  
  if(!mix->open_soundcard()) {
    error("soundcard not found, exiting.");
    exit(1);
  }
    
  Amuse *gui = new Amuse(310, 70, PACKAGE);
  init_callbacks(mix,gui);
  
  mix->register_gui(gui);
  
  gui->start();
  
  while(!mix->quit) {
    mix->cafudda();
  }
  
  notice("quitting MuSE");
  jsleep(1,0);

  delete mix;
  act("cya!");
  
  exit(0);
}
