/* playlist.cpp - MuSE
   (c)2000 Denis Roio aka jaromil
   see COPYING for licensing stuff

   playlist container class
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <jutils.h>
#include <playlist.h>
#include <generic.h>

Entry::Entry(char *file) {
  name = strdup(file);
}

Entry::~Entry() {
  free(name);
}

Playlist::Playlist() {
  length = 0;
  first = NULL;
  selected = NULL;
  last = NULL;
}

Playlist::~Playlist() {
  clear();
}

void Playlist::add(char *file) {
  Entry *ptr;

  if(first==NULL) { /* that's the first entry */
    first = new Entry(file);
    first->next = NULL;
    first->prev = NULL;
    last = first;
  } else { /* add the entry to the end */
    ptr = first;
    while(ptr->next!=NULL) ptr = ptr->next;
    ptr->next = new Entry(file);
    ptr->next->next = NULL;
    ptr->next->prev = ptr;
    last = ptr->next;
  }
  length++;
}
  
void Playlist::clear() {
  if(length>0) {
    Entry *ptr = first, *tptr;
    int c;
    free(filename);
    for(c=0;c<length;c++) {
      tptr = ptr->next;
      delete ptr;
      ptr = tptr;
    }
    length = 0;
    first = NULL;
    selected = NULL;
    last = NULL;
  }
      
}

bool Playlist::load(char *file) {
  char temp[MAX_PATH_SIZE], *p;
  int c = 0;
  FILE *fd;

  clear();

  fd = fopen(file, "r");
  if(fd==NULL)
    return(false);
  while(fgets(temp,MAX_PATH_SIZE,fd)!=NULL) {
    for(p = temp;*p!='\n';p++);

    if(*p=='\n') *p = '\0';
    else {
      warning("UPS! invalid filename :(");
      act("try to substitute null-spaces and be sure it's not longer than %u chars",MAX_PATH_SIZE);
      return(false);
    }
    add(temp);
    c++;
  }
  fclose(fd);
  if(c>0) {
    filename = strdup(file);
    length = c;
  }
  return(true);
}

char *Playlist::song(int pos) {
  if((length<pos)||(pos<=0)) return(NULL);
  if(pos==1) return(first->name);
  if(pos==length) return(last->name);
  
  Entry *ptr = first;
  for(int c=1;c<pos;c++) {
    if(ptr->next == NULL)
      return(NULL);
    ptr = ptr->next;
  }
  return(ptr->name);
}

void Playlist::select(int pos) {
  if((length<pos)||(pos<=0)) selected = NULL;
  if(pos==1) selected = first;
  if(pos==length) selected = last;

  Entry *ptr = first;
  int c;
  
  for(c=1;c<pos;c++) {
    if(ptr->next == NULL)
      return;
    ptr = ptr->next;
  }
  selected = ptr;
}

Entry *Playlist::pick(int pos) {
  if((length<pos)||(pos==0)) return(NULL);
  if(pos==1) return(first);
  if(pos==length) return(last);

  Entry *ptr = first;
  int c;
  for(c=1;c<pos;c++) ptr = ptr->next;

  return(ptr);
}

bool Playlist::moveup(int pos) {
  if(pos<=1) return(false);
  Entry *p = pick(pos);
  Entry *prev = p->prev, *next = p->next, *pp = prev->prev;
  
  if(next!=NULL) next->prev = prev;
  p->next = prev;
  p->prev = pp;
  prev->next = next;
  prev->prev = p;
  if(pp!=NULL) pp->next = p;
  
  if(pos==2) first = p;
  if(pos==length) last = prev;
  return(true);
}

bool Playlist::movedown(int pos) {
  if(pos>=length) return(false);
  Entry *p = pick(pos);
  Entry *prev = p->prev, *next = p->next, *nn = next->next;

  if(prev!=NULL) prev->next = next;
  p->prev = next;
  p->next = nn;
  next->prev = prev;
  next->next = p;
  if(nn!=NULL) nn->prev = p;
  
  if(pos==length-1) last = p;
  if(pos==1) first = next;
  return(true);
}
